package oliot.aika.object;

import oliot.aika.rajapinta.AikaRajapinta;

/**
 * Luokka toteuttamaan sovitun julkisen rajapinnan ja Object-
 * luokan metodeja
 * @author  Vesa Lappalainen @version 1.0, 01.02.2003
 * @author Santtu Viitanen @version 1.1, 9.7.2011
 * @example
 * @example
 * <pre name="test">
 *  Aika a1 = new Aika(12,15);
 *  a1.lisaa(55);  a1.toString() === "13:10";
 *  a1.aseta(12,15); a1.toString() === "12:15";
 * </pre>
 */
public class Aika implements AikaRajapinta,Cloneable {

    private int h, m;


    /**
     * Alustaa ajan muotoon 00:00
     * @example
     * <pre name="test">
     * new Aika().toString() === "00:00";
     * </pre>
     */
    public Aika() {
        aseta(0, 0);
    }


    /**
     * Alustaa ajan
     * @param h tunnit
     * @param m minuutit
     * @example
     * <pre name="test">
     * new Aika(12,15).toString() === "12:15";
     * </pre>
     */
    public Aika(int h, int m) {
        aseta(h, m);
    }


    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
     * @param ih asetettava tuntimr
     * @param im asetettava minuuttimr
     * @return montako vuorokautta ji yli
     * <pre name="test">
     *  Aika a1 = new Aika();
     *  a1.aseta(12,15); a1.toString() === "12:15"; 
     *  a1.aseta(15,45); a1.toString() === "15:45";
     *  a1.aseta(-49,-125); a1.toString() === "20:55";
     * </pre>
     */
    @Override
    public final int aseta(int ih, int im) {
        int th = ih;
        int tm = im;
        th += tm / 60; // liiat minuutit tunteihin
        tm %= 60; // minuutit ville -59 - 59
        int vrk = th / 24; // liiat tunnit vuorokausiin
        th %= 24; // tunnit vlille 0-23

        if (tm < 0) {
            tm += 60;
            th--;
        } // negatiiviset arvot
        if (th < 0) {
            th += 24;
            vrk--;
        }
        this.h = th; // asetetaan lasketut arvot attribuutteihin
        this.m = tm;
        return vrk; // motako vuorokautta ji yli tai ali
    }


    /**
     * Lis aikaan valitun minuuttimrn
     * @param lisaMin listtv minuuttimr
     * @return ylimriset vuorokaudet
     * @example
     * <pre name="test">
     *  Aika a1 = new Aika(13,16);
     *  a1.lisaa(27);    a1.toString() === "13:43";
     *  a1.lisaa(39);    a1.toString() === "14:22";
     * </pre>
     */
    @Override
    public int lisaa(int lisaMin) {
        return aseta(h, m + lisaMin);
    }


    /**
     * Lis aikaan valitun minuuttimrn
     * @param aika aika jota listn 
     * @param lisaMin listtv minuuttimr
     * @example
     * <pre name="test">
     *  Aika a1 = new Aika(13,16);
     *  lisaa(a1,27);    a1.toString() === "13:43";
     *  lisaa(a1,39);    a1.toString() === "14:22";
     * </pre>
     */
    public static void lisaa(Aika aika, int lisaMin) {
        int yht_min = aika.h * 60 + aika.m + lisaMin;
        aika.h = yht_min / 60;
        aika.m = yht_min % 60;
    }


    @Override
    public int getH() {
        return h;
    }


    @Override
    public int getM() {
        return m;
    }


    /**
     * @return aika merkkijonona muodossa 12:05
     * @example
     * <pre name="test">
     * new Aika(12,15).toString() === "12:15";
     * </pre>
     */
    @Override
    public String toString() {
        return String.format("%02d:%02d", getH(), getM());
    }


    /**
     * @example
     * <pre name="test">
     * Aika a1 = new Aika(13,37);
     * Aika a2 = new Aika(13,37);
     * a1 === a2
     * </pre>
     */
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof AikaRajapinta)) return false;
        AikaRajapinta a = (AikaRajapinta) o;
        return getH() == a.getH() && getM() == a.getM();
    }


    /**
     * @example
     * <pre name="test">
     * #THROWS CloneNotSupportedException
     * Aika aika = new Aika(13,37);
     * Aika aika2 = aika.clone();
     * aika2.toString() === "13:37";
     * </pre>
     */
    @Override
    public Aika clone() {
        Aika a = null;
        try {
            a = (Aika)super.clone();
            // a.aseta(h,m); // tt ei tarvitse, koska super laittaa int yms. attr.
        } catch (CloneNotSupportedException e) { // 
        }
        return a;
    }


    /**
     * Aika sekunteina vuorokauden alusta
     * @example
     * <pre name="test">
     * new Aika(13,37).hashCode() === 49020;
     * </pre>
     */
    @Override
    public int hashCode() {
        return 3600 * getH() + 60 * getM();
    }


    @Override
    public int aseta(int ih) {
        return aseta(ih,0);
    }
    
}
