package oliot.aika.rajapinta.perinta;


import oliot.aika.rajapinta.Aika;

/**
 * Luokka toteuttamaan sovitun julkisen rajapinnan
 * @author  Vesa Lappalainen @version 1.0, 01.02.2003
 * @author Santtu Viitanen @version 1.1, 8.7.2011
 * @example
 * <pre name="test">
 * AikaSek a4 = new AikaSek(14,55,45);
 * a4.toString() === "14:55:45";
 * a4.lisaa(3,30); a4.toString() === "14:59:15";
 * AikaSek a5 = new AikaSek(); a5.toString() === "00:00:00";
 * AikaSek a6 = new AikaSek(12); a6.toString() === "12:00:00";
 * AikaSek a7 = new AikaSek(12,15); a7.toString() === "12:15:00"; 
 * </pre>
 */
public class AikaSek extends Aika {

    private int s;


    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
     * @param ih asetettavat tunnit
     * @param im asetettavat minuutit
     * @param is asetettavat sekunnit
     * @return yli- tai alijneet vuorokaudet
     * @example
     * <pre name="test">
     *   AikaSek a = new AikaSek();
     *   a.aseta(10,15,20) === 0; a.toString() === "10:15:20";
     *   a.aseta(25,15,20) === 1; a.toString() === "01:15:20";
     * </pre>
     */
    public final int aseta(int ih, int im, int is) {
        int tm = im;
        int ts = is;
        tm += ts / 60; //liiat sekunnit minuutteihin
        ts %= 60; //sekunnit vlille -59 - 59
        if (ts < 0) {
            ts += 60;
            tm--;
        }
        int vrk = aseta(ih, tm);
        this.s = ts;
        return vrk;
    }


    /**
     * Alustetaan keskiyhn
     * @example
     * <pre name="test">
     *   AikaSek a = new AikaSek();
     *   a.toString() === "00:00:00";
     * </pre>
     */
    public AikaSek() {
        aseta(0, 0, 0);
    }


    /**
     * Alustaa ajan
     * @param h tunnit alustettavat tunnit
     * @param m minuutit alustettavat minuutit
     * @param s sekunnit alustettavat sekunnit
     * @example
     * <pre name="test">
     *   AikaSek a = new AikaSek(12,30,20);
     *   a.toString() === "12:30:20"; 
     * </pre>
     */
    public AikaSek(int h, int m, int s) { // Muodostaja
        aseta(h, m, s);
    }


    /**
     * Alustetaan tunnit ja minuutit, sekunnit 0
     * @param h tunnit alustettavat tunnit
     * @param m minuutit alustettavat minuutit
     * @example
     * <pre name="test">
     *   AikaSek a = new AikaSek(12,30);
     *   a.toString() === "12:30:00"; 
     * </pre>
     */
    public AikaSek(int h, int m) {
        aseta(h, m, 0);
        // tai: this(h,m,0);
    }


    /**
     * Alustetaan tunnit, minuutit ja sekunnit 0
     * @param h tunnit alustettavat tunnit
     * @example
     * <pre name="test">
     *   AikaSek a = new AikaSek(12);
     *   a.toString() === "12:00:00"; 
     * </pre>
     */
    public AikaSek(int h) {
        aseta(h, 0, 0);
    }


    /**
     * @return aika merkkijonona muodossa 12:05:03
     * @example
     * <pre name="test">
     *   AikaSek a = new AikaSek(12,05,03);
     *   a.toString() === "12:05:03"; 
     * </pre>
     */
    @Override
    public String toString() {
        return super.toString() + String.format(":%02d", getS());
    }


    /**
     * Lis aikaan valitun minuuttimrn
     * @param lisaMin listtv minuuttimr
     * @param lisaSek listtv sekunttimr
     * @return ylimenneet vuorokaudet
     * @example
     * <pre name="test">
     *   AikaSek a = new AikaSek(12,50,30);
     *   a.lisaa(11,40)    === 0; a.toString() === "13:02:10";
     *   a.lisaa(12*60,40) === 1; a.toString() === "01:02:50";
     *   AikaSek as = new AikaSek(12,50,30);
     *   as.lisaa(11)    === 0; as.toString() === "13:01:30";
     *   as.lisaa(12*60) === 1; as.toString() === "01:01:30";
     * </pre>
     */
    public int lisaa(int lisaMin, int lisaSek) {
        return aseta(getH(), getM() + lisaMin, this.s + lisaSek);
    }


    /**
     * @return ajan sekunnit
     * @example
     * <pre name="test">
     *   AikaSek a = new AikaSek(12,50,30);
     *   a.getS() === 30;
     * </pre>
     */
    public int getS() {
        return s;
    }


    /**
     * @param args ei kytss
     */
    public static void main(String[] args) {
        Aika a1 = new Aika();
        Aika a2 = new Aika(13);
        Aika a3 = new Aika(14, 175);
        AikaSek a4 = new AikaSek(14, 55, 45);

        // Rajapintaan perustuva esimerkki polymorfisesta taulukosta
        Aika ajat[] = new Aika[5];
        ajat[0] = a1;
        ajat[1] = a2;
        ajat[2] = a3;
        ajat[3] = a4;
        ajat[4] = new AikaSek(23, 59, 59);

        for (int i = 0; i < ajat.length; i++) {
            System.out.print(ajat[i] + " +" + i + " => ");
            ajat[i].lisaa(i);
            System.out.println(ajat[i]);
        }
    }
}
