package siivoa;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import fi.jyu.mit.ohj2.Tiedosto;

/**
 * Ohjelma, jolla siivotaan Wordin tekem HTML-tiedostoa.
 * Tehty Office Word 2010 versiota varten, tarvittaessa muutettava
 * jos tulostusmuoto muuttuu.
 * Tallenna Wodista HTML-tiedosto (Web page)
 * Tallenna Wordist WebPage filtered (nimelle moniste.htm)
 * Tee tiedosto basic.sed jossa aina tarvittavat sed-vaihdot
 * tyyliin
 *   s//&auml;/g
 *   s//&ouml;/g
 * ja tietysti css-tyylien laittaminen vaikka </head> etupuolelle
 *   s!</head>!<omacsss></head>!  
 * Sitten aja tm ohjelma parametreill: webnimi moniste.htm
 * Tuloksena syntyy
 *  m.html  - se iso filtterimtn Word versio, sisllys korvattu paremmalla
 *  moniste.html - filtterity word versio, sisllys korvattu paremmalla
 *  toc.html - vasempaan frameen tuleva sisllysluettelon osa.
 *  toc.sed - sed skripti (aja sed -f moniste.html) nimien vaihtamiseksi paremmaksi
 * 
 * @author vesal
 * @version 25.12.2011
 *
 */
public class Siivoa {

    /** Kuinka linkit alkavat */
    static final String msoToc = "<p class=MsoToc";
    static final String msoTof = "<p class=MsoTof"; 
    
    static final String TARGET = "target=\"content\"";
    static final String FILENAME = "moniste.html";
    
    /**
     * Luokka yhden sisllysluettelon rivin hoitamiseksi
     * @author vesal
     */
    public static class SisallysAlkio {
        private int taso=0;
        private String linkki="";
        private String teksti="";

        /**
         * Oletusmuodostaja tyhjill alustamiseksi
         */
        public SisallysAlkio() {
            // ei tarvitse tehd mitn
        }

        
        /**
         * Alustetaan alkio
         * @param taso mik on linkin syvyys, 1 = perus
         * @param linkki suhteellinen linkkiviite
         * @param teksti linkin kohdalle tulostuva teksti
         */
        public SisallysAlkio(int taso, String linkki, String teksti) {
            this.taso = taso;
            this.linkki = linkki;
            this.teksti = teksti;
        }


        /**
         * @return sisllysalkion tasonro
         */
        public int getLevel() {
            return taso;
        }
        
        
        /**
         * @return tolppaerotettu merkkijono
         * @example
         * <pre name="test">
         *   SisallysAlkio t = new SisallysAlkio(3,"alku","Aloitus");
         *   t.toString() === "3|alku|Aloitus";
         * </pre>
         */
        @Override
        public String toString() {
            return taso + "|" + linkki + "|" + teksti; 
        }
        

        /**
         * Linkkikorvaamista varten linkiksi tehdn sislt kuvaava teskti,
         * josta poistetaan mahdolliset luku, kuva tehtv yms numerot
         * jotta linkki ei muutu numeroinnin muuttuessa.
         * @return sed muotoinen jono linkin korvaamiseksi kuvaavalla jonolla
         * @example
         * <pre name="test">
         *   SisallysAlkio t = new SisallysAlkio(1,"_Toc1234","1.1.1  Eka ali   luku 12");
         *   t.toLink() === "Eka_ali_luku";
         *   t = new SisallysAlkio(1,"_Toc1234","Tehtv 3.1 Kvelyohjeet");
         *   t.toLink() === "Tehtava_Kavelyohjeet";
         *   t = new SisallysAlkio(1,null,"Tehtv 3.1 Kvelyohjeet");
         *   t.toLink() === "";
         * </pre>
         */
        public String toLink() {
            if ( linkki == null ) return "";
            String s = teksti.replaceFirst("^[0-9.]+\\s*", ""); // numerot pois alusta
            s = s.replaceAll("\\s[0-9]*$", "");    // numerot pois lopusta
            String linkText = poistaTurhat(s);
            linkText = linkText.replaceAll("^Tehtava[_0-9]+", "Tehtava_");
            linkText = linkText.replaceAll("^Kuva[_0-9]+", "Kuva_");
            return linkText;
        }

        
        /**
         * Linkkikorvaamista varten linkiksi tehdn sislt kuvaava teskti,
         * josta poistetaan mahdolliset luku, kuva tehtv yms numerot
         * jotta linkki ei muutu numeroinnin muuttuessa.
         * @return sed muotoinen jono linkin korvaamiseksi kuvaavalla jonolla
         * @example
         * <pre name="test">
         *   SisallysAlkio t = new SisallysAlkio(1,"_Toc1234","1.1.1  Eka ali   luku 12");
         *   t.toSedString() === "s/_Toc1234/Eka_ali_luku/g";
         *   t = new SisallysAlkio(1,"_Toc1234","Tehtv 3.1 Kvelyohjeet");
         *   t.toSedString() === "s/_Toc1234/Tehtava_Kavelyohjeet/g";
         *   t = new SisallysAlkio(1,null,"Tehtv 3.1 Kvelyohjeet");
         *   t.toSedString() === "";
         * </pre>
         */
        public String toSedString() {
            if ( linkki == null ) return "";
            return "s/"+linkki+"/"+toLink()+"/g";
        }

        
        /**
         * @return lajiteltava linkkijono, jossa aluperisteksi mukana
         * @example
         * <pre name="test">
         *   SisallysAlkio t = new SisallysAlkio(1,"_Toc1234","1.1.1  Eka ali   luku 12");
         *   t.toLinkString() === "Eka_ali_luku|1.1.1  Eka ali   luku 12: _Toc1234";
         *   t = new SisallysAlkio(1,"_Toc1234","Tehtv 3.1 Kvelyohjeet");
         *   t.toLinkString() === "Tehtava_Kavelyohjeet|Tehtv 3.1 Kvelyohjeet: _Toc1234";
         *   t = new SisallysAlkio(1,null,"Tehtv 3.1 Kvelyohjeet");
         *   t.toLinkString() === "";
         * </pre>
         */
        public String toLinkString() {
            if ( linkki == null ) return "";
            return toLink() + "|" + teksti + ": " + linkki;
        }

        
        /**
         * Html-muotoinen rivi linkiksi muutettuna
         * @param nextLevel seuraavan linkin taso
         * @param target jono joka listn  
         * @return html-muotoinen jono, josta puuttuu </li>, jonka joutuu kutsuva laittamaan
         * @example
         * <pre name="test">
         *   SisallysAlkio t = new SisallysAlkio(1,"_Toc1234","1.1.1  Eka ali   luku");
         *   String target = "target=\"content\""; 
         *   t.toHtmlString(2,target) === "<li class=\"exp\"><a target=\"content\" href=\"moniste.html#_Toc1234\">1.1.1 Eka ali luku</a>"
         *   t.toHtmlString(1,target) === "<li class=\"basic\"><a target=\"content\" href=\"moniste.html#_Toc1234\">1.1.1 Eka ali luku</a>"
         *   t.toHtmlString(2,"") === "<li class=\"exp\"><a href=\"#_Toc1234\">1.1.1 Eka ali luku</a>"
         *   t.toHtmlString(1,null) === "<li class=\"basic\"><a href=\"#_Toc1234\">1.1.1 Eka ali luku</a>"
         *   t = new SisallysAlkio(1,null,"Kuvat:");
         *   t.toHtmlString(2,target) === "<li class=\"exp\">Kuvat:";
         * </pre>
         */
        public String toHtmlString(int nextLevel,String target) {
            String tasoString = "basic";
            if ( nextLevel > taso ) tasoString = "exp";
            String siivottuTeksti = teksti.replaceAll("\\s+", " ");
            if ( linkki == null ) 
                return "<li class=\""+tasoString+"\">"+siivottuTeksti;
            
            String starget = target;
            if (starget == null ) starget = "";
            String filename = "";
            if ( starget.length() > 0 ) {
                starget += " ";
                filename = FILENAME;
            }
            return "<li class=\""+tasoString+"\"><a " + starget + "href=\""+ filename + "#"+linkki+"\">"+siivottuTeksti+"</a>";
        }

        
        private static final Pattern patternMatcher = Pattern.compile(
                ".*<a\\s*href=\"([^\"]*)\">(.*)</.*");
        private static final Pattern tofPatternMatcher = Pattern.compile(
                ".*PAGEREF ([^\\s]+)\\s.*");
        
        /**
         * Ottaa linkin merkkijonosta
         * @param s jono josta linkki otetaan
         * @return pelkk linkki, null jos ei saa linkki
         * @example
         * <pre name="test">
         *   String s = SisallysAlkio.takeLink("<p class=MsoToc1><span style='mso'></span><a href=\"#linkki\"><span...><span>Lukijalle</span></a>...");
         *   s === "linkki";
         *   s = SisallysAlkio.takeLink("<p class=MsoToc1><span style='mso'></span><a href=\"linkki\"><span...><span><b>Luki</b>jalle</span></a>...");
         *   s === "linkki";
         * </pre>
         */
        public static String takeLink(String s) {
            Matcher m = patternMatcher.matcher(s);
            if ( !m.matches() ) return null;

            String link = m.group(1); 
            return link.replaceAll("#","");
        }

        
        /**
         * Ottaa taso linkin merkkijonosta
         * @param s jono josta linkki otetaan
         * @return tason numero
         * @example
         * <pre name="test">
         *   int n = SisallysAlkio.takeTaso("<p class=MsoToc1><span style='mso'></span><a href=\"linkki\"><span...><span>Lukijalle</span></a>...");
         *   n === 1;
         *   n = SisallysAlkio.takeTaso("<p class=MsoToc2><span style='mso'></span><a href=\"linkki\"><span...><span>Lukijalle</span></a>...");
         *   n === 2;
         *   SisallysAlkio.takeTaso("kana") === 0;
         *   SisallysAlkio.takeTaso("<p class=MsoToc") === 0;
         * </pre>
         */
        public static int takeTaso(String s) {
            int i = s.indexOf(msoToc);
            if ( i < 0 ) return 0;
            int i2 = i+msoToc.length();
            if ( i2 >= s.length() ) return 0;
            return s.charAt(i2)-'0';
        }

        
        /**
         * Paluatetaan 1. > < vliss oleva osa jossa on jotakin muuta
         * kuin tyhj.  Oletus ett jonossa on tllainen esiintym 
         * @param s jono jossa on html-koodia
         * @return lopttavan ja aloittavan vliss olevien jonojen yhdiste
         * @example
         * <pre name="test">
         *   SisallysAlkio.pureText("><span...><span>luki</span>aa<><>ss<") === "lukiaass";
         *   SisallysAlkio.pureText("><span...><span>luki<b>jalle</b></span>aa<><>ss<") === "lukijalleaass";
         *   SisallysAlkio.pureText("><span...><span>luki<b>jalle...</b>") === "lukijalle";
         * </pre>
         */
        public static String pureText(String s) {
            int alku = -1;
            String text = "<>"+s+"<>";
            text = text.replaceAll("<!--.*?-->", "");
            StringBuilder tulos = new StringBuilder();
            for (int i=0; i<text.length(); i++) {
                if ( text.charAt(i) == '<' ) {
                    if (alku >= 0 && alku < i) tulos.append(text.substring(alku,i));
                    alku = -1;
                }
                if ( text.charAt(i) == '>' ) alku = i+1;
            }
            String result = tulos.toString().trim();
            return result.replaceAll("\\.*$", "");
        }
        
        
        /**
         * Kaivetaan linkkitieto merkkijonosta
         * @param s jono josta linkkiteitoa etsitn
         * @return saatiinko kaikki tarpeellinen tieto
         * @example
         * <pre name="test">
         *   SisallysAlkio t = new SisallysAlkio();
         *   t.parse("kana") === false;
         *   String s = "<p class=MsoToc1><span style='mso-bookmark:_Toc377107274'>..."+
         *              " style='mso-bookmark:_Toc377107274'></span><a href=\"#_Toc312598587\"><span"+
         *              " style='mso-bookmark:_Toc377107274'><span lang=EN-GB>1 Lukijalle</span>. 1.2</span><span"+
         *              " mso-ansi-language:FI;mso-fareast-language:FI'><o:p></o:p></span></span></p>";
         *   t.parse(s) === true;   
         *   t.toString() === "1|_Toc312598587|1 Lukijalle";           
         * </pre>
         */
        public boolean parse(String s) {
            int uusitaso = takeTaso(s);
            if ( uusitaso <= 0 ) return false;
            Matcher m = patternMatcher.matcher(s);
            if ( !m.matches() ) return false;
            String uusilinkki = m.group(1).replaceAll("#",""); 
            String text = m.group(2);
            this.teksti = pureText(text).replaceAll("\\s+[.0-9]*$", "");
            this.teksti = this.teksti.replaceAll("[.]*$", "");
            this.linkki = uusilinkki;
            this.taso = uusitaso;
            return true;
        }
        
        
        /**
         * Kaivetaan tof-linkkitieto merkkijonosta
         * @param s jono josta linkkiteitoa etsitn
         * @return saatiinko kaikki tarpeellinen tieto
         * @example
         * <pre name="test">
         *   SisallysAlkio t = new SisallysAlkio();
         *   t.parse("kana") === false;
         *   String s = "<p class=MsoTof><p class=MsoTof><span style='ms'>"+
         *              "<span style='mso'>Tehtv 3.1</span>"+
         *              "<span style='mso-tab-count:1'></span> </span>Kvelyohjeet"+
         *              "<span style='mso-tab-count:1 dotted'>.... </span>"+
         *              "<span style='mso-bookmark:_Toc377107274'> PAGEREF _Toc312753613 \\h <span>"+
         *              "<span>28</span></p>";
         *   t.parseTof(s) === true;   
         *   t.toString() === "2|_Toc312753613|Tehtv 3.1 Kvelyohjeet";           
         * </pre>
         */
        public boolean parseTof(String s) {
            int uusitaso = 2;
            Matcher m = tofPatternMatcher.matcher(s);
            if ( !m.matches() ) return false;
            String text = pureText(s);
            text = text.replaceAll("\\.\\.+", "");
            this.linkki = m.group(1);
            teksti = text.replaceAll("\\sPAGEREF\\s"+this.linkki+".*\\\\h", "");
            teksti = teksti.replaceAll("\\s+", " ");
            teksti = teksti.replaceAll("\\s+[0-9]*$", "");
            char c = 160; // tt on jostakin syyst koodissa
            teksti = teksti.replaceAll("[\\s"+c+"]+$", "");
            this.taso = uusitaso;
            return true;
        }


    }

    
    /**
     * Tehdn HTML-muotoinen toc
     * @param toc SisallysAlkioiden lista
     * @param target target-jono sisllykseen
     * @return toc html-merkkijonona
     * @example
     * <pre name="test">
     *   List<SisallysAlkio> toc = new ArrayList<SisallysAlkio>();
     *   toc.add(new SisallysAlkio(1,"_Toc312598587","1 Lukijalle")); 
     *   toc.add(new SisallysAlkio(2,"_Toc312598588","1.2 Aluksi")); 
     *   toc.add(new SisallysAlkio(1,"1","eka")); 
     *   toc.add(new SisallysAlkio(1,"11","eka1"));
     *   List<String> htoc = teeHtmlToc(toc,"target=\"content\""); 
     *   int i=0;
     *   htoc.get(i++) === "<ul class=\"subexp\">";
     *   htoc.get(i++) === "<li class=\"exp\"><a target=\"content\" href=\"moniste.html#_Toc312598587\">1 Lukijalle</a>"; 
     *   htoc.get(i++) === "<ul class=\"sub\">";
     *   htoc.get(i++) === "<li class=\"basic\"><a target=\"content\" href=\"moniste.html#_Toc312598588\">1.2 Aluksi</a>"; 
     *   htoc.get(i++) === "</li>";
     *   htoc.get(i++) === "</ul>";
     *   htoc.get(i++) === "</li>";
     *   htoc.get(i++) === "<li class=\"basic\"><a target=\"content\" href=\"moniste.html#1\">eka</a>"; 
     *   htoc.get(i++) === "</li>";
     *   htoc.get(i++) === "<li class=\"basic\"><a target=\"content\" href=\"moniste.html#11\">eka1</a>"; 
     *   htoc.get(i++) === "</li>";
     *   htoc.get(i++) === "</ul>";
     * </pre>
     */
    public static List<String> teeHtmlToc(List<SisallysAlkio> toc,String target) {
        List<String> htoc = new ArrayList<String>(); 
        htoc.add("<ul class=\"subexp\">");
        int lastLevel = -1;
        int openedLevel = -1;

        for (int ti = 0; ti < toc.size(); ti++) {
                SisallysAlkio t = toc.get(ti);
                int nextLevel = 0;
                if ( ti+1 < toc.size()) nextLevel = toc.get(ti+1).getLevel();
                lastLevel = t.getLevel();
                if ( lastLevel > openedLevel && openedLevel > 0) {
                    htoc.add("<ul class=\"sub\">");
                    openedLevel = lastLevel;
                }
                else {
                    if ( openedLevel > 0 ) {
                        for ( int lev = openedLevel; lev >= lastLevel+1; lev--) {
                            htoc.add("</li>");
                            htoc.add("</ul>");
                        }
                        htoc.add("</li>");
                    }
                    openedLevel = lastLevel;
                }
                htoc.add(t.toHtmlString(nextLevel,target));
        }
        for ( int lev = lastLevel; lev >= 1+1; lev--) {
            htoc.add("</li>");
            htoc.add("</ul>");
        }
        htoc.add("</li>");
        htoc.add("</ul>");
        return htoc;        
    }
    
    
    /**
     * Etsitn sisllyluettelo (table of contents) Word dokumentista
     * @param html lista, jossa koko html-tiedosto
     * @return lista, jossa sisllysluettelon osuus
     * @example
     * <pre name="test">
     * #import java.util.ArrayList;
     * #import java.util.List;
     *   ArrayList<String>html = new ArrayList<String>();
     *   html.add("<div class=WordSection2><w:Sdt SdtDocPart=\"t\" DocPartType=\"Table of Contents\"");
     *   html.add("DocPartUnique=\"t\" ID=\"615721169\">");
     *   html.add("<p class=MsoToc1><span style='mso-bookmark:_Toc377107274'>...");
     *   html.add(" style='mso-bookmark:_Toc377107274'></span><a href=\"#_Toc312598587\"><span");
     *   html.add(" style='mso-bookmark:_Toc377107274'><span lang=EN-GB>1 Lukijalle</span> 1</span><span");
     *   html.add("  mso-ansi-language:FI;mso-fareast-language:FI'><o:p></o:p></span></span></p>");
     *   html.add("<p class=MsoToc2><span style='mso-bookmark:_Toc377107274'>...");
     *   html.add(" style='mso-bookmark:_Toc377107274'></span><a href=\"#_Toc312598588\"><span");
     *   html.add(" style='mso-bookmark:_Toc377107274'><span lang=EN-GB>1.2 Aluksi</span></span><span");
     *   html.add("  mso-ansi-language:FI;mso-fareast-language:FI'><o:p></o:p></span></span></p>");
     *   html.add("<p class=MsoToc1><a href=\"#1\">eka</a></p>"); 
     *   html.add("<p class=MsoToc1><a href=\"#11\">eka1</a></p>"); 
     *   html.add("</w:Sdt>"); 
     *   html.add("...");
     *   List<SisallysAlkio> toc = etsiToc(html);
     *   int i=0;
     *   toc.get(i++).toString() === "1|_Toc312598587|1 Lukijalle"; 
     *   toc.get(i++).toString() === "2|_Toc312598588|1.2 Aluksi"; 
     *   toc.get(i++).toString() === "1|1|eka"; 
     *   toc.get(i++).toString() === "1|11|eka1"; 
     * </pre>
     */
    public static List<SisallysAlkio> etsiToc(List<String> html) {
        List<SisallysAlkio> toc = new ArrayList<SisallysAlkio>();
        StringBuilder sb = new StringBuilder();

        for (String s:html) {
            if ( s.contains("</w:Sdt>") ) break;
            int i = s.indexOf(msoToc); 
            if ( i >= 0 ) {
                sb.delete(0, 100000);
                sb.append(s);
            }
            else if ( sb.length() > 0 ) sb.append(s); 
            if ( sb.length() > 0 && s.contains("</p>") ) {
                SisallysAlkio t = new SisallysAlkio();
                if ( t.parse(sb.toString()) ) 
                    toc.add(t);
                sb.delete(0, 100000);
            }
        }
        return toc;
    }


    /**
     * Etsitn sisllyluettelon kuvat (table of figures) Word dokumentista
     * @param html lista, jossa koko html-tiedosto
     * @return lista, jossa kuvasisllysluettelon osuus
     * @example
     * <pre name="test">
     * #import java.util.ArrayList;
     * #import java.util.List;
     *   ArrayList<String>html = new ArrayList<String>();
     *   html.add("<div class=WordSection2><w:Sdt SdtDocPart=\"t\" DocPartType=\"Table of Contents\"");
     *   html.add("DocPartUnique=\"t\" ID=\"615721169\">");
     *   html.add("<p class=HeadingCSis><span style='mso-bookmark:_Toc377107274'>Tehtvt:</span></p>");
     *   html.add("<p class=MsoTof>Tehtv 3.1 Kvelyohjeet  PAGEREF _Toc13 \\h 28</p>");
     *   html.add("<p class=MsoTof>Tehtv 3.2    Muita");
     *   html.add("lajittelualgoritmeja  PAGEREF _Toc14 \\h 29</p>");
     *   html.add("<p class=HeadingCSis><span style='mso-bookmark:_Toc377107274'></span><!--[if supportFields]><span");
     *   html.add("style='mso-bookmark:_Toc377107274'></span><span style='mso-element:field-end'></span><![endif]--><span");
     *   html.add("style='mso-bookmark:_Toc377107274'>Kuvat:</span></p>");
     *   html.add("<p class=MsoTof>");
     *   html.add("Kuva 4.1 Ehtolauseet  PAGEREF _Toc15 \\h 29</p>");
     *   html.add("</div>");
     *   List<SisallysAlkio> toc = etsiTof(html);
     *   int i=0;
     *   toc.get(i++).toString() === "1|null|Tehtvt:"; 
     *   toc.get(i++).toString() === "2|_Toc13|Tehtv 3.1 Kvelyohjeet"; 
     *   toc.get(i++).toString() === "2|_Toc14|Tehtv 3.2 Muita lajittelualgoritmeja"; 
     *   toc.get(i++).toString() === "1|null|Kuvat:"; 
     *   toc.get(i++).toString() === "2|_Toc15|Kuva 4.1 Ehtolauseet"; 
     * </pre>
     */
    public static List<SisallysAlkio> etsiTof(List<String> html) {
        List<SisallysAlkio> tof = new ArrayList<SisallysAlkio>();
        StringBuilder sb = new StringBuilder();
        boolean tofStarted = false;
        
        for (String s:html) {
            if ( tofStarted && s.contains("</div>") ) break;
            int i = s.indexOf(msoTof);
            if ( i < 0 ) i = s.indexOf("<p class=HeadingCSis");
            if ( i >= 0 ) {
                sb.delete(0, 100000);
                sb.append(s);
            }
            else if ( sb.length() > 0 ) {
                if ( sb.charAt(sb.length()-1) != '>' ) sb.append(" "); // Jos rivi on katkennut 
                sb.append(s); 
            }
            if ( sb.length() > 0 && s.contains("</p>") ) {
                if ( sb.indexOf("<p class=HeadingCSis") >= 0 ) {
                    String text = SisallysAlkio.pureText(sb.toString());
                    text = text.replaceAll("\\s+", " ").trim();
                    tof.add(new SisallysAlkio(1,null,text));
                } else {
                    SisallysAlkio t = new SisallysAlkio();
                    if ( t.parseTof(sb.toString()) ) 
                        tof.add(t);
                }
                sb.delete(0, 100000);
            }
        }
        return tof;
    }


    /**
     * Poistaa jono turhat merkit ja vihata skandit aakkoiksi
     * @param jono korjattava jono
     * @return korjattu jono
     * @example
     * <pre name="test">
     *   poistaTurhat("  kissa  (koti)") === "kissa";
     *   poistaTurhat("  kkk  ") === "kaakka";
     *   poistaTurhat("  kkk???  ") === "kaakka";
     *   poistaTurhat("  kkk! % !\" <kana> \\a 1+2 3-4 4/5") === "kaakka_a_1_2_3_4_4_5";
     *   poistaTurhat("aku ankka.") === "aku_ankka";
     *   poistaTurhat("aku    ankka [iines]") === "aku_ankka";
     *   poistaTurhat("K&auml;&auml;nt&auml;j&auml;n") === "Kaantajan";
     *   poistaTurhat("C# C++") === "Csharp_Cpp";
     *   poistaTurhat("1<2") === "1lt2";
     *   poistaTurhat("1>=2") === "1gteq2";
     *   poistaTurhat("A&#8208;B") === "A_B";
     * </pre>
     */
    public static String poistaTurhat(String jono)
    {
        String[][] korvattavat = {
                                 {"\t"," "},         // tab merkit pois
                                 {"&#8208;","_"},    // pitk viiva
                                 {"&#8211;","_"},    // pitk viiva
                                 {"&#[0-9]*;","_"},  // muut koodeillaolevat erikoismerkit
                                 {"&quot;","_"},     // "
                                 {"#","sharp"},      // C# => Csharp
                                 {"\\+\\+","pp"},     // C++ => Cpp
                                 {"\\([^)]*\\)",""},  // () oleva osa pois
                                 {"\\[[^\\]]*\\]",""}, // [] oleva osa pois
                                 {"\\{[^}]*\\}",""},  // {} oleva osa pois
                                 {"\\<[^>]*>",""},   // <> oleva osa pois
                                 {"\\<","lt"},       // < => lt
                                 {"\\>","gt"},       // < => gt
                                 {"==","equals"},   // = => eq
                                 {"!=","notequals"},// = => notequals
                                 {"=","eq"},        // = => eq
                                 {"","a"},          // skandit aakkosiksi
                                 {"","o"},
                                 {"","A"},
                                 {"","O"},
                                 {"&auml;","a"},
                                 {"&ouml;","o"},
                                 {"&Auml;","A"},
                                 {"&Ouml;","O"},
                               //  {@"[\/-\+\.=\&\!]+","_"},
                                 {"[+&!-.<>=/]+","_"},  // erikoismerkeist tyhj
                                 {"[^_a-z0-9A-Z ]+",""}, // muut erikoismerkit pois
                                 {"  +", " "},           // monesta vlist yksi
                                 {" ", "_"},             // jljelle jneist alleviiva
                                 {"_+", "_"},            // monesta perkkisest _ yksi
                                 {"^_", ""},             // alussa olevat _ pois
                                 {"_$", ""},             // samoin lopussa
                                 };
        String tulos = jono;
        for (int i = 0; i < korvattavat.length; i++)
            tulos = tulos.replaceAll(korvattavat[i][0], korvattavat[i][1]);

        return tulos;
    }


    
    /**
     * Poistaa taulukosta ensimmisen vlin, joka alkaa jonolla s1 ja loppuun jonoon s2 
     * @param html lista josta poistetaan
     * @param s1 jono josta poisto aloitetaan, jos ei lydy, ei poisteta mitn
     * @param s2 jono, johon poisto lopetetaan, jos ei lydy, ei poisteta mitn
     *                 jos s2 == null etsitn vain s1 esiintym
     * @param alkuoffset paljonko poisto siirretn alun lytymkohdasta
     * @param loppuoffset paljonko loppu siirretn lopun lytymkohdasta
     * @return palauttaa rivin jolta poisto alkoi, -1 jos mitn ei poistettu
     * @example
     * <pre name="test">
     *   ArrayList<String>html = new ArrayList<String>();
     *   html.add("0 aaa");
     *   html.add("1 aba");
     *   html.add("2 aca");
     *   html.add("3 ade");
     *   html.add("4 ada");
     *   poista(html,"ab","ca",0,1) === 1;
     *   html.get(1) === "4 ada"; 
     *   poista(html,"ab","ca",0,0) === -1;
     *   poista(html,"ad","da",0,0) === 1;
     *   html.size() === 1;
     *   poista(html,"aa",null,0,100000000) === 0;
     *   html.size() === 0;
     * </pre>
     */
    public static int poista(List<String> html, String s1, String s2, int alkuoffset, int loppuoffset) {
        int start = -1;
        int i = -1;
        for (String s : html) {
            i++;
            if ( start < 0 && s.contains(s1) ) start = i;
            if ( start >= 0 && ( s2 == null || s.contains(s2) ) ) {
               if ( start == i && s1.equals(s2) ) continue;
               start += alkuoffset; if ( start < 0 ) start = 0;
               int end = i + loppuoffset;
               if ( end >= html.size() ) end = html.size()-1;
               if ( end < start ) end = start;
               html.subList(start,end+1).clear();
               break;
            }
        }
        return start;
    }

    
    /**
     * Luetaan html-tiedosto ja etsitn siit sisllysviitteet ja tehdn
     * sed-skripti joka korvaa viitteet paremmilla.  Tuotetaan mys
     * kauniimpi sisllysluettelo. 
     * @param args nimi1 nimi2
     * @throws IOException jos stdin "rikki"
     */
    public static void main(String[] args) throws IOException {
        if ( args.length == 0 ) {
            System.out.println("Anna parametriksi Word-html tiedoston nimi");
            return;
        }


        List<String> html = Tiedosto.lueTiedostoListaan(args[0]);
        if ( html == null ) {
            System.out.println("Tiedostoa " + args[0] + " ei voi lukea");
            return;
        }

        List<SisallysAlkio> toc = etsiToc(html);
        List<SisallysAlkio> tof = etsiTof(html);
        List<String> htoc = teeHtmlToc(toc,TARGET);
        List<String> htof = teeHtmlToc(tof,TARGET);
        htoc.addAll(htof);
        Tiedosto.kirjoitaTiedosto("toc.html", htoc);

        
        List<String> sedList = new ArrayList<String>();
        List<String> linkList = new ArrayList<>();
        List<String> basicSed = Tiedosto.lueTiedostoListaan("basic.sed"); 
        //sedList.add("s!\\(<link rel=OLE-Object-Data href=\"ohj2moniste_files/oledata.mso\">\\)!\\1"+
        /*
        sedList.add("s!</head>!"+
                "<link rel=\"stylesheet\" type=\"text/css\" href=\"../skriptit/treestyles.css\" />"+
                "<link type=\"text/css\" rel=\"stylesheet\" href=\"../skriptit/omat.css\" />"+
                "<script type=\"text/javascript\" src=\"../skriptit/marktree.js\"></script>"+
                "</head>!");
                */
        if ( basicSed != null ) sedList.addAll(basicSed);
        
        for (SisallysAlkio t : toc) { sedList.add(t.toSedString()); linkList.add(t.toLinkString()); }
        for (SisallysAlkio t : tof) { sedList.add(t.toSedString()); linkList.add(t.toLinkString()); }
        
        Tiedosto.kirjoitaTiedosto("toc.sed", sedList);
        
        int start = poista(html,"DocPartType=\"Table of Contents\"","</div>",0,0);
        htoc.clear();
        htoc.add("<h1>Sisllys</h1>");
        htoc.addAll(teeHtmlToc(toc,""));
        htof = teeHtmlToc(tof,"");
        htoc.addAll(htof);
        html.addAll(start, htoc);
        Tiedosto.kirjoitaTiedosto("m.html", html);
        
        if ( args.length > 1 ) {
            html = Tiedosto.lueTiedostoListaan(args[1]);
            if ( html == null ) {
                System.out.println("Tiedostoa " + args[1] + " ei voi lukea");
                return;
            }
            start = poista(html,"<div class=WordSection2>","</div>",0,0);
            if ( start < 0 ) return;
            html.addAll(start, htoc);
            poista(html,"<table class=MsoTableGrid","<table class=MsoTableGrid",0,-1);
            Tiedosto.kirjoitaTiedosto(FILENAME, html);
        }        

        Collections.sort(linkList);
        String prevLink = "";
        String prevS = "";
        int samoja = 0;
        for (String s:linkList) {
            int p = s.indexOf('|');
            if ( p < 0 ) continue;
            String currentLink = s.substring(0,p);
            if ( prevLink.equalsIgnoreCase(currentLink) ) {
                System.out.println(prevS);
                System.out.println(s);
                samoja++;
            }
            prevLink = currentLink;
            prevS = s;
        } 
        if ( samoja > 0 ) {
            System.out.println("Warning: " + samoja + " equal links");
            System.in.read();
        }
        
        
    }

    
}
