package fi.jyu.mit.gui;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JSplitPane;

import fi.jyu.mit.ohj2.IniFile;
import fi.jyu.mit.ohj2.Mjonot;


/**
 * Luokka Swing-pohjaisten komponenttien tietojen tallettamiseen ini-tiedostoon.
 * <pre>
 * [pos]
 * MyGUI=10,10,200,100
 * [settings]
 * splitPanePaikka=30
 * </pre>
 * 
 * 
 * @author vesal
 * @version 15.6.2013
 * 
 * @example
 * <pre name="testJAVA">
 * #import java.io.File;
 * #import fi.jyu.mit.ohj2.IniFile;
 *   private static File fini = new File("testini.ini");
 *   private static File fini2 = new File("testini2.ini");
 *   private IniFile ini = IniFile.create("testini.ini");
 *   @BeforeClass 
 *   public static void alusta() {
 *      fini.delete();  fini2.delete();
 *   }   
 *   
 *   
 *   // fini.renameTo(fini2) === true; // koska saman luominen ei loisi mitn
 *   
 *   @AfterClass
 *   public static void siivoa() {
 *      fini.delete();  fini2.delete();
 *   }   
 * </pre>
 * 
 */
public class IniSwing  {

    /**
     * Kirjoitetaan suorakaiteen tiedot ini-tiedostoon 
     * @param ini mit ini-tiedostoa kytetn
     * @param section mihin osioon kirjoitetaan
     * @param item mik on nimi
     * @param value mik on uusi arvo
     * @example
     * <pre name="test">
     *#import java.awt.Rectangle;
     *    Rectangle rect = new Rectangle(20,20,250,150);
     *    write(ini,"settings","rect",rect);
     *    Rectangle r2 = read(ini,"settings","rect",null);
     *    r2.x === rect.x;
     *    r2.y === rect.y;
     *    r2.width === rect.width;
     *    r2.height === rect.height;
     * </pre>
     */
    public static void write(IniFile ini, String section, String item, Rectangle value) {
        String pos = String.format("%d,%d,%d,%d",value.x,value.y,value.width,value.height);
        ini.write(section,item,pos);
    }
    
    
    /**
     * Luetaan suorakaiteen tiedot ini-tiedostosta
     * @param ini mit ini-tiedostoa kytetn
     * @param section mist osioista luetaan
     * @param item mist kohdasta luetaan
     * @param def mik on oletustiedot jos ei lydy
     * @return luettun suorakaiteen tiedot
     */
    public static Rectangle read(IniFile ini,String section, String item, Rectangle def) {
        String pos = ini.read(section,item);
        if ( pos == null ) return def;
        String[] posa = (pos+",,,").split(",");
        int x = Mjonot.erotaInt(posa[0], -1); if ( x < 0 ) return def;
        int y = Mjonot.erotaInt(posa[1], -1); if ( y < 0 ) return def;
        int w = Mjonot.erotaInt(posa[2], -1); if ( w < 0 ) return def;
        int h = Mjonot.erotaInt(posa[3], -1); if ( h < 0 ) return def;
        return new Rectangle(x,y,w,h);
    }


    /**
     * Ottaa komponentin koon ja laittaa sen ini-tiedostoon
     * @param ini mit ini-tiedostoa kytetn
     * @param comp mink komponentin koko
     * @param section mist osioista luetaan
     * @param item mist kohdasta luetaan
     */
    public static void writeBounds(IniFile ini, Component comp, String section, String item) {
        final String itemname = getItemName(comp,item);
        Rectangle rect = comp.getBounds();
        write(ini,section,itemname,rect);
    }


    /**
     * Asettaa ikkunan koon
     * @param ini mit ini-tiedostoa kytetn
     * @param comp mink komponentin koko
     * @param section mist osioista luetaan
     * @param item mist kohdasta luetaan
     */
    public static void readBounds(IniFile ini, Component comp, String section, String item) {
        final String itemname = getItemName(comp,item);
        Rectangle pos = read(ini,section,itemname,comp.getBounds());
        comp.setBounds(pos);
    }


    /**
     * Ksittelee ikkunan kokoon liittyvt asiat. Aluksi lukee
     * komponentin paikan ja koon ini-tiedostosta.  
     * Sen jlkeen tallentaa ne tiedostoon
     * jos komponentti liikkuu tai muuttaa kokoa
     * @param ini mit ini-tiedostoa kytetn
     * @param comp mit komponenttia ksitelln
     * @param section mist osioista luetaan
     * @param item mist kohdasta luetaan, jos null, kytetn componentin nime
     * @example
     * <pre name="test">
     *#import javax.swing.JFrame;
     * JFrame f = new JFrame();
     * f.setName("frame");
     * handleBounds(ini,f,"pos",null);
     * f.setSize(230,110);
     * f.setLocation(30,40); // ei valitettavasti laukaise testiajossa componentMoved
     * writeBounds(ini,f,"pos",null); // siksi pit itse kirjoittaa
     * f = null;
     * f = new JFrame();
     * f.setName("frame");
     * handleBounds(ini,f,"pos",null);
     * Rectangle rect = f.getBounds();
     * rect.toString() === "java.awt.Rectangle[x=30,y=40,width=230,height=110]";
     * </pre>
     */
    public static void handleBounds(final IniFile ini, final Component comp, final String section, final String item) {
        final String itemname = getItemName(comp,item);
        readBounds(ini, comp, section, itemname);
        comp.addComponentListener(new ComponentAdapter() {
            @Override
            public void componentMoved(ComponentEvent e) {
                writeBounds(ini, e.getComponent(),section,itemname);
            }
            @Override
            public void componentResized(ComponentEvent e) {
                writeBounds(ini, e.getComponent(),section,itemname);
            }
        });
    }


    /**
     * Ottaa komponentin item-tiedot. 
     * @param comp mist komponentista
     * @param item mit rivi kyten ini-tiedostossa, jos null, niin otetaan komponentin nimi
     * @return joko item tai kompoentin nimi
     */
    private static String getItemName(Component  comp, String item) {
        if ( item != null ) return item;
        String name = comp.getName();
        return name;
    }


    /**
     * Ksitelln splitPanen erottimen paikan tiedot.
     * Alukis lukee vanhan paikan ini-tiedostosta ja asettaa sen.
     * Sitten jos paikaa muutetaan, tallentaa tiedot ini-tiedostoon
     * @param ini mit ini-tiedostoa kytetn
     * @param splitPane mink splitPanen
     * @param section mist osiosta luetaan
     * @param item mit rivi kyten ini-tiedostossa, jos null, niin otetaan komponentin nimi
     * @param pos alkuperinen paikka
     * @example
     * <pre name="test">
     *#import javax.swing.JSplitPane;
     *    JSplitPane sp = new JSplitPane();
     *    handleSplitPane(ini,sp,"split","paikka1",25);
     *    sp.setDividerLocation(40);
     *    sp = null;
     *    sp = new JSplitPane();
     *    sp.setName("paikka1");
     *    handleSplitPane(ini,sp,"split",null,25);
     *    sp.getDividerLocation() === 40;
     * </pre>
     */
    public static void handleSplitPane(final IniFile ini, final JSplitPane splitPane, final String section, final String item, int pos) {
         final String itemname = getItemName(splitPane,item);
         splitPane.setDividerLocation(ini.read(section, itemname, pos));
         splitPane.addPropertyChangeListener(JSplitPane.DIVIDER_LOCATION_PROPERTY,new PropertyChangeListener() {
             @Override
             public void propertyChange(PropertyChangeEvent evt) {
                 int w = splitPane.getDividerLocation();
                 ini.write(section,itemname, w);
             }
         });
    }
}
