package fi.jyu.mit.gui;

import java.util.Arrays;

/**
 * Luokka kokonaislukujen silyttmiseksi renkaassa
 * Jos listn liikaa alkioita, vanhin poistetaan.
 * @author vesal
 * @version 10.2.2013
 */
public class IntRing {
    private int maxN;
    private int n = 0;
    private int[] items = null;
    
    
    /**
     * Alustetaan taulukko
     * @param maxN maksimimr. Jos < 0 ei rajaa
     */
    public IntRing(int maxN) {
        this.maxN = maxN;
        if ( maxN < 0 ) return;
        items = new int[maxN];
    }
    
    
    /**
     * @return alkiot merkkijonona
     * @example
     * <pre name="test">
     *   IntRing r = new IntRing(-1);
     *   r.toString() === "null";
     *   r = new IntRing(4);
     *   r.toString() === "" 
     *   r.add(3); r.add(2); r.add(0);
     *   r.toString() === "3 2 0";
     * </pre>
     */
    @Override
    public String toString() {
        if ( items == null ) return "null";
        StringBuilder result = new StringBuilder();
        String sep = "";
        for (int i=0; i<n; i++) {
            result.append(sep+items[i]);
            sep = " ";
        }
        return result.toString();
    }
    
    
    /**
     * Listn alkio.
     * @param i listtv alkio
     * @return MIN_VALUE jos mahtui, muuten poistettu arvo
     * @example
     * <pre name="test">
     *   IntRing r = new IntRing(3);
     *   r.add(3) === Integer.MIN_VALUE;
     *   r.add(2) === Integer.MIN_VALUE;
     *   r.add(0) === Integer.MIN_VALUE;
     *   r.toString() === "3 2 0";
     *   r.add(1) === 3;   r.toString() === "2 0 1";
     *   r.add(5) === 2;   r.toString() === "0 1 5";
     *   r.add(5) === 0;   r.toString() === "1 5 5";
     * </pre>
     */
    public int add(int i) {
        if ( items == null ) return Integer.MIN_VALUE;
        int result = Integer.MIN_VALUE;
        if ( n >= maxN ) result = removeInd(0);
        items[n++] = i;
        return result;
    }

    
    /**
     * Listn alkio tai jos se jo on, siirretn se viimeiseksi
     * @param i listtv alkio
     * @return MIN_VALUE jos mahtui, muuten poistettu arvo
     * @example
     * <pre name="test">
     *   IntRing r = new IntRing(3);
     *   r.add1(3) === Integer.MIN_VALUE;
     *   r.add1(2) === Integer.MIN_VALUE;
     *   r.add1(0) === Integer.MIN_VALUE;
     *   r.toString() === "3 2 0";
     *   r.add1(1) === 3;   r.toString() === "2 0 1";
     *   r.add1(5) === 2;   r.toString() === "0 1 5";
     *   r.add1(0) === Integer.MIN_VALUE;  r.toString() === "1 5 0";
     *   r.add1(2) === 1;   r.toString() === "5 0 2";
     * </pre>
     */
    public int add1(int i) {
        if ( items == null ) return Integer.MIN_VALUE;
        int idx = indexOf(i);
        if ( idx >= 0 ) removeInd(idx);
        return add(i);
    }

    
    /**
     * Poistetaan 1.  arvo jos lytyy
     * @param value poistettava arvo
     * @return value jos poistettiin, muuten MIN_VALUE
     * @example
     * <pre name="test">
     *   IntRing r = new IntRing(3);
     *   r.add(3); r.add(2); r.add(0);
     *   r.indexOf(2) === 1; 
     *   r.indexOf(5) === -1;
     *   r.indexOf(3) === 0;   
     * </pre>
     */
    public int indexOf(int value) {
        for (int i=0; i<n; i++)
            if ( items[i] == value ) return i;
        return -1;
    }

    
    /**
     * Poistetaan 1.  arvo jos lytyy
     * @param value poistettava arvo
     * @return value jos poistettiin, muuten MIN_VALUE
     * @example
     * <pre name="test">
     *   IntRing r = new IntRing(3);
     *   r.add(3); r.add(2); r.add(0);
     *   r.remove(2) === 2;   r.toString() === "3 0";
     *   r.remove(5) === Integer.MIN_VALUE;   r.toString() === "3 0";
     *   r.remove(3) === 3;   r.toString() === "0";
     *   r.remove(0) === 0;   r.toString() === "";
     *   r.remove(5) === Integer.MIN_VALUE;   r.toString() === "";
     * </pre>
     */
    public int remove(int value) {
        int i = indexOf(value);
        if ( i >= 0 ) return removeInd(i);
        return Integer.MIN_VALUE;
    }

    
    private int removeInd(int idx) {
        if ( idx < 0 || n <= idx  ) return Integer.MIN_VALUE;
        int result = items[idx];
        for (int i=idx; i<n-1; i++)
            items[i] = items[i+1];
        n--;        
        return result;
    }
    
    
    /**
     * Tyhjent silin kokonaan
     * @example
     * <pre name="test">
     *   IntRing r = new IntRing(3);
     *   r.add(3); r.add(2); r.add(0);
     *   r.clear();   r.toString() === "";
     * </pre>
     */
    public void clear() {
         n = 0;
    }

    
    /**
     * @param idx paikka jossa oleva arvo palautetaan
     * @return arvo paikassa idx
     * @example
     * <pre name="test">
     *   IntRing r = new IntRing(3);
     *   r.add(3); r.add(2); r.add(0);
     *   r.get(0) === 3; 
     *   r.get(1) === 2;
     *   r.get(2) === 0;
     *   r.get(4) === Integer.MIN_VALUE;
     * </pre>
     */
    public int get(int idx) {
        if ( idx < 0 || n <= idx  ) return Integer.MIN_VALUE;
        return items[idx];
    }
    
    
    /**
     * @return ensimminen listan arvo
     * @example
     * <pre name="test">
     *   IntRing r = new IntRing(3);
     *   r.getFirst() === Integer.MIN_VALUE;
     *   r.add(3); r.getFirst() === 3;
     *   r.add(2); r.getFirst() === 3;
     *   r.remove(3); r.getFirst() === 2;
     * </pre>
     */
    public int getFirst() {
        if ( n <= 0 ) return Integer.MIN_VALUE;
        return items[0];
    }
    
    
    /**
     * @return viimeinen listan arvo
     * @example
     * <pre name="test">
     *   IntRing r = new IntRing(3);
     *   r.getLast() === Integer.MIN_VALUE;
     *   r.add(3); r.getLast() === 3;
     *   r.add(2); r.getLast() === 2;
     *   r.remove(2); r.getLast() === 3;
     * </pre>
     */
    public int getLast() {
        if ( n <= 0 ) return Integer.MIN_VALUE;
        return items[n-1];
    }
    
    
    /**
     * @return alkioiden mr taulukossa
     * @example
     * <pre name="test">
     *   IntRing r = new IntRing(3);
     *   r.count() === 0;
     *   r.add(3); r.count() === 1; 
     *   r.add(2); r.count() === 2;
     *   r.add(0); r.count() === 3;
     *   r.add(4); r.count() === 3;
     * </pre>
     */
    public int count() {
        return n;
    }
    
    
    /**
     * @return kopio alkioista
     * @example
     * <pre name="test">
     * #import java.util.Arrays;
     *   IntRing r = new IntRing(-1);
     *   r.getItems() === null;
     *   r = new IntRing(3);
     *   Arrays.toString(r.getItems()) === "[]";
     *   r.add(3); Arrays.toString(r.getItems()) === "[3]"; 
     *   r.add(2); Arrays.toString(r.getItems()) === "[3, 2]";
     *   r.add(0); Arrays.toString(r.getItems()) === "[3, 2, 0]";
     *   r.add(4); Arrays.toString(r.getItems()) === "[2, 0, 4]";
     * </pre>
     */
    public int[] getItems() {
        if ( items == null ) return null;
        return Arrays.copyOf(items, n);
    }
    
}
